package com.blogspot.revbingo.threed;

import java.awt.Color;
import java.awt.Graphics2D;

/**
 * Represents a point in 3D space
 * 
 * @author mark
 * 
 */
public class Point {

	// Make these public, getters and setters will just get in the way
	public double x;
	public double y;
	public double z;

	/**
	 * Construct a point in 3D space
	 * 
	 * @param x
	 * @param y
	 * @param z
	 */
	public Point(double x, double y, double z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}

	public void translate(double dx, double dy, double dz) {
		x += dx;
		y += dy;
		z += dz;
	}
	
	public void rotateZ(double angle) {
		double x0 = x;
		x = (x0 * Math.cos(angle) - y * Math.sin(angle));
		y = (y * Math.cos(angle) + x0 * Math.sin(angle));
	}

	public void rotateX(double angle) {
		double y0 = y;
		y = (y0 * Math.cos(angle) - z * Math.sin(angle));
		z = (z * Math.cos(angle) + y0 * Math.sin(angle));
	}

	public void rotateY(double angle) {
		double z0 = z;
		z = (z0 * Math.cos(angle) - x * Math.sin(angle));
		x = (x * Math.cos(angle) + z0 * Math.sin(angle));
	}

	public void rotate(double angle, RotationAxis axis) {
		switch (axis) {
		case X:
			rotateX(angle);
			break;
		case Y:
			rotateY(angle);
			break;
		case Z:
			rotateZ(angle);
			break;
		}
	}

	/**
	 * Points will be drawn as a single pixel. The z component is ignored,
	 * a.k.a. parallel projection
	 */
	public void draw(Graphics2D g) {
		g.setColor(Color.WHITE);
		g.drawLine((int) x, (int) y, (int) x, (int) y);
	}

	public Vector vectorTo(Point p) {
		Vector v = new Vector(p.x - this.x, p.y - this.y, p.z - this.z);
		return v;
	}

	public String toString() {
		return "Point " + x + "," + y + "," + z;
	}
}
