package com.blogspot.revbingo.threed.lights;

import java.awt.Color;

import com.blogspot.revbingo.threed.Lightable;
import com.blogspot.revbingo.threed.Point;
import com.blogspot.revbingo.threed.Vector;

public class Spotlight extends Light {

	private double fullIntensityAngle;
	private double falloffAngle;
	private double cosFullSpread;
	private float colorHue;
	private float colorSaturation;
	private float colorBrightness;
	
	public Spotlight(Color col, Point position, Point pointsAt, double fullIntensityAngle, double falloffAngle){
		this.color = col;
		
		float[] components = Color.RGBtoHSB(col.getRed(), col.getGreen(), col.getBlue(), null);
		colorHue = components[0];
		colorSaturation = components[1];
		colorBrightness = components[2];
		this.position = position;
		setFocus(pointsAt);
		
		this.fullIntensityAngle = fullIntensityAngle;
		this.falloffAngle = falloffAngle;
		cosFullSpread = Math.cos((fullIntensityAngle + falloffAngle) * (Math.PI/180));
	}

	@Override
	public Color light(Lightable s) {
		Color litColor = Color.BLACK;
		
		double dtFace = s.getNormal().normalise().dotProduct(lightNormal);
		if(dtFace >= 0) return Color.BLACK;
					
		Point lightSource = this.getPosition();
		Vector lightToPoly = lightSource.vectorTo(s.getPosition());		
		double dtPosition = lightToPoly.normalise().dotProduct(lightNormal);
		if(dtPosition < cosFullSpread) return Color.BLACK;
		
		double angle = Math.acos(dtPosition) * (180/Math.PI);

		double fullSpreadAngle = fullIntensityAngle + falloffAngle;
		double falloffFactor = 1;
		if (angle >= fullIntensityAngle && angle <= fullSpreadAngle) {
			falloffFactor -= ((angle - fullIntensityAngle)/(falloffAngle));
		} 
		litColor = Color.getHSBColor(colorHue, colorSaturation, (float) (Math.abs(dtFace) * falloffFactor * colorBrightness));
		return litColor;
	}

	public double getFullIntensityAngle() {
		return fullIntensityAngle;
	}

	public void setFullIntensityAngle(double fullIntensityAngle) {
		if(fullIntensityAngle >= 0) {
			this.fullIntensityAngle = fullIntensityAngle;
		}
		cosFullSpread = Math.cos((fullIntensityAngle + falloffAngle) * (Math.PI/180));
	}

	public double getFalloffAngle() {
		return falloffAngle;
	}

	public void setFalloffAngle(double falloffAngle) {
		if(falloffAngle >= 0) {
			this.falloffAngle = falloffAngle;
		}
		cosFullSpread = Math.cos((fullIntensityAngle + falloffAngle) * (Math.PI/180));
	}

}
